@@ -20,15 +20,12 @@ package org.apache.felix.framework.util;
 
 import java.io.*;
 import java.lang.reflect.*;
-import java.net.JarURLConnection;
-import java.net.MalformedURLException;
 import java.net.*;
 import java.security.*;
 import java.util.Hashtable;
 import java.util.jar.JarFile;
 
-import org.apache.felix.framework.searchpolicy.ContentClassLoader;
-import org.apache.felix.framework.searchpolicy.ContentLoaderImpl;
+import org.apache.felix.framework.searchpolicy.ModuleImpl;
 import org.osgi.framework.BundleActivator;
 import org.osgi.framework.BundleContext;
 
@@ -518,15 +515,15 @@ public class SecureAction
         }
     }
 
-    public JarFile openJAR(File file, boolean verify) throws IOException
+    public JarFileX openJAR(File file, boolean verify) throws IOException
     {
         if (System.getSecurityManager() != null)
         {
             try
             {
                 Actions actions = (Actions) m_actions.get();
-                actions.set(Actions.OPEN_JAR_ACTION, file, (verify ? Boolean.TRUE : Boolean.FALSE));
-                return (JarFile) AccessController.doPrivileged(actions, m_acc);
+                actions.set(Actions.OPEN_JARX_VERIFY_ACTION, file, (verify ? Boolean.TRUE : Boolean.FALSE));
+                return (JarFileX) AccessController.doPrivileged(actions, m_acc);
             }
             catch (PrivilegedActionException ex)
             {
@@ -539,16 +536,17 @@ public class SecureAction
         }
         else
         {
-            return new JarFileX(file);
+            return new JarFileX(file, verify);
         }
     }
-
-    public ContentClassLoader createContentClassLoader(ContentLoaderImpl impl)
+// TODO: REFACTOR - SecureAction fix needed.
+/*
+    public ModuleClassLoader createModuleClassLoader(ModuleImpl impl)
     {
-        return createContentClassLoader(impl, null);
+        return createModuleClassLoader(impl, null);
     }
 
-    public ContentClassLoader createContentClassLoader(ContentLoaderImpl impl,
+    public ModuleClassLoader createModuleClassLoader(ModuleImpl impl,
         ProtectionDomain protectionDomain)
     {
         if (System.getSecurityManager() != null)
@@ -556,8 +554,8 @@ public class SecureAction
             try
             {
                 Actions actions = (Actions) m_actions.get();
-                actions.set(Actions.CREATE_CONTENTCLASSLOADER_ACTION, impl, protectionDomain);
-                return (ContentClassLoader) AccessController.doPrivileged(actions, m_acc);
+                actions.set(Actions.CREATE_MODULECLASSLOADER_ACTION, impl, protectionDomain);
+                return (ModuleClassLoader) AccessController.doPrivileged(actions, m_acc);
             }
             catch (PrivilegedActionException ex)
             {
@@ -566,10 +564,10 @@ public class SecureAction
         }
         else
         {
-            return new ContentClassLoader(impl, protectionDomain);
+            return new ModuleClassLoader(impl, protectionDomain);
         }
     }
-
+*/
     public void startActivator(BundleActivator activator, BundleContext context)
         throws Exception
     {
@@ -614,28 +612,6 @@ public class SecureAction
         }
     }
 
-    public void exit(int code)
-    {
-        if (System.getSecurityManager() != null)
-        {
-            try
-            {
-                Actions actions = (Actions) m_actions.get();
-                actions.set(Actions.SYSTEM_EXIT_ACTION, new Integer(code));
-                AccessController.doPrivileged(actions, m_acc);
-            }
-            catch (PrivilegedActionException ex)
-            {
-                // We don't need to rethrow anything since System.exit throws
-                // runtime exceptions only
-            }
-        }
-        else
-        {
-            System.exit(code);
-        }
-    }
-
     public Policy getPolicy()
     {
         if (System.getSecurityManager() != null)
@@ -657,22 +633,22 @@ public class SecureAction
         }
     }
 
-    public void addURLToURLClassLoader(URL extension, ClassLoader loader) throws Exception 
+    public void addURLToURLClassLoader(URL extension, ClassLoader loader) throws Exception
     {
-        if (System.getSecurityManager() != null) 
+        if (System.getSecurityManager() != null)
         {
             Actions actions = (Actions) m_actions.get();
             actions.set(Actions.ADD_EXTENSION_URL, extension, loader);
-            try 
+            try
             {
                 AccessController.doPrivileged(actions, m_acc);
-            } 
-            catch (PrivilegedActionException e) 
+            }
+            catch (PrivilegedActionException e)
             {
                 throw e.getException();
             }
         }
-        else 
+        else
         {
             Method addURL =
                 URLClassLoader.class.getDeclaredMethod("addURL",
@@ -681,7 +657,7 @@ public class SecureAction
             addURL.invoke(loader, new Object[]{extension});
         }
     }
-    
+
     public Constructor getConstructor(Class target, Class[] types) throws Exception
     {
         if (System.getSecurityManager() != null)
@@ -702,7 +678,7 @@ public class SecureAction
             return target.getConstructor(types);
         }
     }
-    
+
     public Method getMethod(Class target, String method, Class[] types) throws Exception
     {
         if (System.getSecurityManager() != null)
@@ -723,7 +699,7 @@ public class SecureAction
             return target.getMethod(method, types);
         }
     }
-    
+
     public Method getDeclaredMethod(Class target, String method, Class[] types) throws Exception
     {
         if (System.getSecurityManager() != null)
@@ -744,7 +720,28 @@ public class SecureAction
             return target.getDeclaredMethod(method, types);
         }
     }
-    
+
+    public void setAccesssible(Method method)
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.SET_ACCESSIBLE_ACTION, method);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw (RuntimeException) e.getException();
+            }
+        }
+        else
+        {
+            method.setAccessible(true);
+        }
+    }
+
     public Object invoke(Method method, Object target, Object[] params) throws Exception
     {
         if (System.getSecurityManager() != null)
@@ -767,6 +764,27 @@ public class SecureAction
         }
     }
     
+    public Object invokeDirect(Method method, Object target, Object[] params) throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_DIRECTMETHOD_ACTION, method, target, params);
+            try
+            {
+                return AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            return method.invoke(target, params);
+        }
+    }
+
     public Object invoke(Constructor constructor, Object[] params) throws Exception
     {
         if (System.getSecurityManager() != null)
@@ -784,12 +802,11 @@ public class SecureAction
         }
         else
         {
-            constructor.setAccessible(true);
             return constructor.newInstance(params);
         }
     }
 
-    public Object getDeclaredField(Class targetClass, String name, Object target) 
+    public Object getDeclaredField(Class targetClass, String name, Object target)
         throws Exception
     {
         if (System.getSecurityManager() != null)
@@ -809,18 +826,18 @@ public class SecureAction
         {
             Field field = targetClass.getDeclaredField(name);
             field.setAccessible(true);
-            
+
             return field.get(target);
         }
     }
 
-    public Object swapStaticFieldIfNotClass(Class targetClazz, 
+    public Object swapStaticFieldIfNotClass(Class targetClazz,
         Class targetType, Class condition, String lockName) throws Exception
     {
         if (System.getSecurityManager() != null)
         {
             Actions actions = (Actions) m_actions.get();
-            actions.set(Actions.SWAP_FIELD_ACTION, targetClazz, targetType, 
+            actions.set(Actions.SWAP_FIELD_ACTION, targetClazz, targetType,
                 condition, lockName);
             try
             {
@@ -833,25 +850,25 @@ public class SecureAction
         }
         else
         {
-            return _swapStaticFieldIfNotClass(targetClazz, targetType, 
+            return _swapStaticFieldIfNotClass(targetClazz, targetType,
                 condition, lockName);
         }
     }
-    
-    private static Object _swapStaticFieldIfNotClass(Class targetClazz, 
+
+    private static Object _swapStaticFieldIfNotClass(Class targetClazz,
         Class targetType, Class condition, String lockName) throws Exception
     {
         Object lock = null;
         if (lockName != null)
         {
-            try 
+            try
             {
-                Field lockField = 
-                    targetClazz.getDeclaredField(lockName); 
+                Field lockField =
+                    targetClazz.getDeclaredField(lockName);
                 lockField.setAccessible(true);
                 lock = lockField.get(null);
-            } 
-            catch (NoSuchFieldException ex) 
+            }
+            catch (NoSuchFieldException ex)
             {
             }
         }
@@ -866,13 +883,13 @@ public class SecureAction
             Object result = null;
             for (int i = 0; (i < fields.length) && (result == null); i++)
             {
-                if (Modifier.isStatic(fields[i].getModifiers()) && 
+                if (Modifier.isStatic(fields[i].getModifiers()) &&
                     (fields[i].getType() == targetType))
                 {
                     fields[i].setAccessible(true);
-                    
+
                     result = fields[i].get(null);
-                    
+
                     if (result != null)
                     {
                         if ((condition == null) ||
@@ -890,7 +907,7 @@ public class SecureAction
                     // reset cache
                     for (int i = 0; i < fields.length; i++)
                     {
-                        if (Modifier.isStatic(fields[i].getModifiers()) && 
+                        if (Modifier.isStatic(fields[i].getModifiers()) &&
                             (fields[i].getType() == Hashtable.class))
                         {
                             fields[i].setAccessible(true);
@@ -907,7 +924,7 @@ public class SecureAction
         }
         return null;
     }
-    
+
     private static class Actions implements PrivilegedExceptionAction
     {
         public static final int GET_PROPERTY_ACTION = 0;
@@ -926,11 +943,11 @@ public class SecureAction
         public static final int DELETE_FILE_ACTION = 13;
         public static final int OPEN_JARX_ACTION = 14;
         public static final int GET_URL_INPUT_ACTION = 15;
-        public static final int CREATE_CONTENTCLASSLOADER_ACTION = 16;
+        public static final int CREATE_MODULECLASSLOADER_ACTION = 16;
         public static final int START_ACTIVATOR_ACTION = 17;
         public static final int STOP_ACTIVATOR_ACTION = 18;
         public static final int SYSTEM_EXIT_ACTION = 19;
-        public static final int OPEN_JAR_ACTION= 20;
+        public static final int OPEN_JARX_VERIFY_ACTION = 20;
         public static final int GET_POLICY_ACTION = 21;
         public static final int CREATE_TMPFILE_ACTION = 22;
         public static final int OPEN_URLCONNECTION_ACTION = 23;
@@ -943,6 +960,8 @@ public class SecureAction
         public static final int SWAP_FIELD_ACTION = 30;
         public static final int GET_FIELD_ACTION = 31;
         public static final int GET_DECLAREDMETHOD_ACTION = 32;
+        public static final int SET_ACCESSIBLE_ACTION = 33;
+        public static final int INVOKE_DIRECTMETHOD_ACTION = 34;
 
         private int m_action = -1;
         private Object m_arg1 = null;
@@ -1005,7 +1024,7 @@ public class SecureAction
 
         public Object run() throws Exception
         {
-            int action = m_action;
+            int action =  m_action;
             Object arg1 = m_arg1;
             Object arg2 = m_arg2;
             Object arg3 = m_arg3;
@@ -1077,19 +1096,19 @@ public class SecureAction
             {
                 return new JarFileX((File) arg1);
             }
-            else if (action == OPEN_JAR_ACTION)
+            else if (action == OPEN_JARX_VERIFY_ACTION)
             {
-                return new JarFile((File) arg1, ((Boolean) arg2).booleanValue());
+                return new JarFileX((File) arg1, ((Boolean) arg2).booleanValue());
             }
             else if (action == GET_URL_INPUT_ACTION)
             {
                 return ((URLConnection) arg1).getInputStream();
             }
-            else if (action == CREATE_CONTENTCLASSLOADER_ACTION)
-            {
-                return new ContentClassLoader((ContentLoaderImpl) arg1,
-                    (ProtectionDomain) arg2);
-            }
+// TODO: REFACTOR - SecureAction fix needed.
+//            else if (action == CREATE_MODULECLASSLOADER_ACTION)
+//            {
+//                return new ModuleClassLoader((ModuleImpl) arg1, (ProtectionDomain) arg2);
+//            }
             else if (action == START_ACTIVATOR_ACTION)
             {
                 ((BundleActivator) arg1).start((BundleContext) arg2);
@@ -1121,7 +1140,7 @@ public class SecureAction
             {
                 return ((JarURLConnection) arg1).getJarFile();
             }
-            else if (action == ADD_EXTENSION_URL) 
+            else if (action == ADD_EXTENSION_URL)
             {
                 Method addURL =
                     URLClassLoader.class.getDeclaredMethod("addURL",
@@ -1142,14 +1161,17 @@ public class SecureAction
                 ((Method) arg1).setAccessible(true);
                 return ((Method) arg1).invoke(arg2, (Object[]) arg3);
             }
+            else if (action == INVOKE_DIRECTMETHOD_ACTION)
+            {
+                return ((Method) arg1).invoke(arg2, (Object[]) arg3);
+            }
             else if (action == INVOKE_CONSTRUCTOR_ACTION)
             {
-                ((Constructor) arg1).setAccessible(true);
                 return ((Constructor) arg1).newInstance((Object[]) arg2);
             }
             else if (action == SWAP_FIELD_ACTION)
             {
-                return _swapStaticFieldIfNotClass((Class) arg1, 
+                return _swapStaticFieldIfNotClass((Class) arg1,
                     (Class) arg2, (Class) arg3, (String) arg4);
             }
             else if (action == GET_FIELD_ACTION)
@@ -1162,8 +1184,12 @@ public class SecureAction
             {
                 return ((Class) arg1).getDeclaredMethod((String) arg2, (Class[]) arg3);
             }
-            
+            else if (action == SET_ACCESSIBLE_ACTION)
+            {
+                ((Method) arg1).setAccessible(true);
+            }
+
             return null;
         }
     }
-}
+}
\ No newline at end of file
