@@ -541,7 +541,7 @@ loop:   for (;;)
     // Exclusive inner classes
     private static final class AndOperator extends Operator
     {
-        private int operandCount;
+        private final int operandCount;
 
         public AndOperator(int opcnt)
         {
@@ -582,12 +582,16 @@ loop:   for (;;)
 
         public void buildTree(Stack operands)
         {
-            children = new Operator[operandCount];
-            // need to preserve stack order
-            for (int i = 0; i < operandCount; i++)
+            if (children == null)
             {
-                children[(operandCount - 1) - i] =
-                    (Operator) operands.pop();
+                Operator[] tmp = new Operator[operandCount];
+                // need to preserve stack order
+                for (int i = 0; i < operandCount; i++)
+                {
+                    tmp[(operandCount - 1) - i] =
+                        (Operator) operands.pop();
+                }
+                children = tmp;
             }
             operands.push(this);
         }
@@ -606,7 +610,7 @@ loop:   for (;;)
 
     private static final class OrOperator extends Operator
     {
-        private int operandCount;
+        private final int operandCount;
 
         public OrOperator(int opcnt)
         {
@@ -647,12 +651,17 @@ loop:   for (;;)
 
         public void buildTree(Stack operands)
         {
-            children = new Operator[operandCount];
-            // need to preserve stack order
-            for (int i = 0; i < operandCount; i++)
+            if (children == null) 
             {
-                children[(operandCount - 1) - i] =
-                    (Operator) operands.pop();
+                Operator[] tmp = new Operator[operandCount];
+            
+                // need to preserve stack order
+                for (int i = 0; i < operandCount; i++)
+                {
+                    tmp[(operandCount - 1) - i] =
+                        (Operator) operands.pop();
+                }
+                children = tmp;
             }
             operands.push(this);
         }
@@ -693,8 +702,11 @@ loop:   for (;;)
 
         public void buildTree(Stack operands)
         {
-            children = new Operator[1];
-            children[0] = (Operator) operands.pop();
+            if (children == null)
+            {
+                children = new Operator[]{
+                    (Operator) operands.pop()};
+            }
             operands.push(this);
         }
 
@@ -787,12 +799,17 @@ loop:   for (;;)
 
         public void buildTree(Stack operands)
         {
-            children = new Operator[2];
-            // need to preserve stack order
-            for (int i = 0; i < 2; i++)
-            {
-                Operator o = (Operator) operands.pop();
-                children[1 - i] = o;
+            if (children == null)
+            { 
+                Operator[] tmp = new Operator[2];
+            
+                // need to preserve stack order
+                for (int i = 0; i < 2; i++)
+                {
+                    Operator o = (Operator) operands.pop();
+                    tmp[1 - i] = o;
+                }
+                children = tmp;
             }
             operands.push(this);
         }
@@ -845,11 +862,16 @@ loop:   for (;;)
 
         public void buildTree(Stack operands)
         {
-            children = new Operator[2];
-            // need to preserve stack order
-            for (int i = 0; i < 2; i++)
+            if (children == null)
             {
-                children[1 - i] = (Operator) operands.pop();
+                Operator[] tmp = new Operator[2];
+            
+                // need to preserve stack order
+                for (int i = 0; i < 2; i++)
+                {
+                    tmp[1 - i] = (Operator) operands.pop();
+                }
+                children = tmp;
             }
             operands.push(this);
         }
@@ -901,11 +923,16 @@ loop:   for (;;)
 
         public void buildTree(Stack operands)
         {
-            children = new Operator[2];
-            // need to preserve stack order
-            for (int i = 0; i < 2; i++)
+            if (children == null)
             {
-                children[1 - i] = (Operator) operands.pop();
+                Operator[] tmp = new Operator[2];
+            
+                // need to preserve stack order
+                for (int i = 0; i < 2; i++)
+                {
+                    tmp[1 - i] = (Operator) operands.pop();
+                }
+                children = tmp;
             }
             operands.push(this);
         }
@@ -957,11 +984,16 @@ loop:   for (;;)
 
         public void buildTree(Stack operands)
         {
-            children = new Operator[2];
-            // need to preserve stack order
-            for (int i = 0; i < 2; i++)
+            if (children == null)
             {
-                children[1 - i] = (Operator) operands.pop();
+                Operator[] tmp = new Operator[2];
+            
+                // need to preserve stack order
+                for (int i = 0; i < 2; i++)
+                {
+                    tmp[1 - i] = (Operator) operands.pop();
+                }
+                children = tmp;
             }
             operands.push(this);
         }
@@ -984,7 +1016,7 @@ loop:   for (;;)
 
     private static final class PresentOperator extends Operator
     {
-        String attribute;
+        final String attribute;
 
         public PresentOperator(String attribute)
         {
@@ -1018,7 +1050,7 @@ loop:   for (;;)
 
     private static final class PushOperator extends Operator
     {
-        String attribute;
+        final String attribute;
 
         public PushOperator(String attribute)
         {
@@ -1061,7 +1093,7 @@ loop:   for (;;)
 
     private static final class ConstOperator extends Operator
     {
-        Object val;
+        final Object val;
 
         public ConstOperator(Object val)
         {
@@ -1098,7 +1130,7 @@ loop:   for (;;)
     private static final class SubStringOperator extends Operator
         implements OperatorConstants
     {
-        String[] pieces;
+        final String[] pieces;
 
         public SubStringOperator(String[] pieces)
         {
@@ -1230,8 +1262,11 @@ loop:   for (;;)
 
         public void buildTree(Stack operands)
         {
-            children = new Operator[1];
-            children[0] = (Operator) operands.pop();
+            if (children == null) 
+            {
+                children = new Operator[]{
+                    (Operator) operands.pop()};
+            }
             operands.push(this);
         }
 
